# API Documentation of Bash Base Library

## error messages

### error-codes

```
error-codes (bash-csv)
# <50 .. reserved for base-base error-codes
#  51 .. illegal handle-name
#  52 .. handle not loaded
#  53 .. line not found
#  54 .. row not found
#  55 .. handle already in use
#  56 .. parse error
#  9x .. errors local to each function
```

### bcsv_string_of_error_code

```
bcsv_string_of_error_code <ret-vn> <error-code>
# returns string representation of error code
# errors: 2, 3
```

## Utils

### csv_delimiter_validate

```
csv_delimiter_validate <delimiter>
```

### bcsv_quote_field

```
bcsv_quote_field <ret-vn> <field> <delimiter>
# returns field, quoted if it contains delimiter characters or quotation marks
```

### bcsv_line_of_row

```
bcsv_line_of_row <ret-vn> <fields-array-vn> <delimiter>
# creates a CSV file line out of a number of fields separated by delimiter
# errors: 1, 2, 3, 7, 11
```

## Handles

### bcsv_is_open

```
bcsv_is_open <handle>
# check if handle is open
# errors: 2, 51, 52
```

### bcsv_new

```
bcsv_new <handle> <default-delimter>
# creates an empty CSV handle
# handle must match ^[[:alpha:]][[:alnum:]_]*$ and must be unused
# error: 2, 51, 55
```

### bcsv_discard

```
bcsv_discard <handle>
# discard ever
# errors, 2, 51, 52
```

## Access and Manipulation of loaded CSVs

### bcsv_delimiter_get

```
bcsv_delimiter_get <ret-vn> <handle>
# returns configured delimiter used by handle
# errors: 2, 3, 51, 52
```

### bcsv_delimiter_set

```
bcsv_delimiter_set <handle> <delimiter>
# sets configured delimiter used by handle
# errors: 2, 3, 51, 52
```

### bcsv_row_size

```
bcsv_row_size <ret-rows-vn> <handle>
# get number of rows
# errors: 2, 3, 51, 52
```

### bcsv_row_get

```
bcsv_row_get <ret-row-array-vn> <handle> <row-number>
# errors: 2, 3, 51, 52, 54
```

### bcsv_row_find

```
bcsv_row_find <ret-row-number-vn> <handle> <column-number> <value>
# returns number of row which has value in column column-number
# errors: 2, 3, 10, 51, 52, 54
```

### bcsv_row_set

```
bcsv_row_set <handle> <row-number> <fields-array-vn>
# set a row, overwriting any potential contents
# if row-number is empty it will behave like bcsv_row_add
# if row-number > current row size empty rows will be filled in
# errors: 2, 3, 51, 52
```

### bcsv_row_set_direct

```
bcsv_row_set_direct <handle> <row-number> [<field> [..]]
# set a row, overwriting any potential contents
# if row-number is empty it will behave like bcsv_row_add
# if row-number > current row size empty rows will be filled in
# errors: 2, 3, 51, 52
```

### bcsv_row_add

```
bcsv_row_add <handle> <field> [..]
# adds a row
# errors: 2, 51, 52
```

## Iterators

### bcsv_row_iter

```
bcsv_row_iter <handle> <iter-fun> [<xarg> [..]]
# iterates over a CSV row by row
# iter-fun must take: <fields-array-vn> <index> [<xarg> [..]]
# errors: 2, 3, 6, 8, 91 (one or more iter-fun calls failed)
```

## CSV Processing

### bcsv_row_map

```
bcsv_row_map <ret-handle> <in-handle> <map-fun> [<xarg>] [..]]
# applys map-fun to every row of a CSV, generating a new CSV as output
# mapfun must take: <ret-fields-array-vn> <fields-array-vn> <index>
#                   [<xarg>] [..]]
# errors: 2, 3, 6, 8, 91 (one or more map-fun calls failed)
```

### bcsv_row_filter

```
bcsv_row_filter <ret-handle> <in-handle> <filter-fun> [<xarg>] [..]]
# create a new CSV containing all rows in source CSV that run filter-fun
# successfully
# filterfun must take: <fields-array-vn> <index> [<xarg>] [..]]
# errors: 2, 3, 6, 8, 91 (one or more filter-fun calls failed)
```

## File IO

### bcsv_load

```
bcsv_load <handle> <delimiter> (<csv-file>|-) [<ret-error-hint-vn>]
# loads contents of csv-file or from stdin if '-' is specified
# handle must match ^[[:alpha:]][[:alnum:]_]*$ and must be unused
# ret-error-hint will be filled with error hint if specified
# errors: 2, 8, 9, 51, 55
```

### bcsv_save

```
bcsv_save <handle> (<csv-file>|-) [<force>]
# saves CSV to file or stdout if none is specified
# if force is non empty existing file will be overwritten
# errors: 2, 9, 51, 52, 91 (file already exists)
```

### bcsv_save_with_delimiter

```
bcsv_save_with_delimiter <handle> <delimiter> (<csv-file>|-) [force]
# saves CSV to file or stdout if none is specified
# if force is non empty existing file will be overwritten
# uses custom delimiter specified
# errors: 2, 9, 51, 52, 91 (file already exists)
```
